@@ -7,7 +7,7 @@
 */
 
 /*
-  $Id: pkgset-install.c,v 1.25 2001/06/24 19:43:02 mis Exp $
+  $Id: pkgset-install.c,v 1.30 2001/07/13 14:49:32 mis Exp $
 */
 
 #include <limits.h>
@@ -18,7 +18,7 @@
 #include <signal.h>
 #include <sys/types.h>
 #include <sys/wait.h>
-
+#include <fnmatch.h>
 
 #include <rpm/rpmlib.h>
 #include <trurl/nassert.h>
@@ -193,7 +193,6 @@ static void reaper (int sig)
 }
 
 
-
 static int runrpm(struct pkgset *ps, struct upgrade_s *upg) 
 {
 #ifndef EXEC_RPM    
@@ -417,13 +416,28 @@ int uninstpkgs_provides(struct upgrade_s *upg, struct capreq *req)
     return 0;
 }
 
-    
 
 static void mapfn_clean_pkg_color(struct pkg *pkg) 
 {
     pkg_set_color(pkg, PKG_COLOR_WHITE);
 }
 
+
+static struct pkg *find_pkg(tn_array *pkgs, const char *name) 
+{
+    struct pkg tmpkg;
+    int i;
+    
+    tmpkg.name = (char*)name;
+
+    n_array_sort(pkgs);
+    i = n_array_bsearch_idx_ex(pkgs, &tmpkg, (tn_fn_cmp)pkg_cmp_name); 
+    if (i < 0)
+        return NULL;
+
+    return n_array_nth(pkgs, i);
+}
+
 #define PROCESS_DEPS       0 
 #define PROCESS_ORPHANS    1
 
@@ -441,6 +455,8 @@ static int process_deps(struct pkgset *ps, tn_array *pkgs,
     
     nloop = 0;
     
+    n_assert(how == PROCESS_DEPS || how == PROCESS_ORPHANS);
+
     while (nmarked > 0) {
         nmarked = 0;
         for (i=0; i<n_array_size(tmparr); i++) {
@@ -452,27 +468,28 @@ static int process_deps(struct pkgset *ps, tn_array *pkgs,
             if (pkg->reqs == NULL)
                 continue;
             
-            msg_i(2, nloop, "%s\n", pkg_snprintf_s(pkg));
+            msg_i(2, nloop, "Checking %s dependencies\n", pkg_snprintf_s(pkg));
 
             for (j=0; j<n_array_size(pkg->reqs); j++) {
                 struct capreq *req;
                 struct pkg **suspkgs, pkgsbuf[1024], *tomark = NULL;
                 int nsuspkgs = 0;
                 char *reqname;
-                int reqnover;
                 
                 req = n_array_nth(pkg->reqs, j);
                 reqname = capreq_name(req);
-                reqnover = capreq_has_ver(req) == 0;
-            
-                if (reqnover && n_hash_exists(upg->depcache, reqname)) {
-                    msg_i(4, nloop, "in cache %s\n", reqname);
+                if (capreq_has_ver(req) != 0) {
+                    reqname = alloca(256);
+                    capreq_snprintf(reqname, 256, req);
+                }
+                
+                if (n_hash_exists(upg->depcache, reqname)) {
+                    msg_i(4, nloop, " in cache %s\n", reqname);
                     continue;
                 }
-
                 
                 if (how == PROCESS_ORPHANS && !uninstpkgs_provides(upg, req)) {
-                    msg(5, "skip %s from %s\n", reqname, pkg_snprintf_s(pkg));
+                    msg(5, " skip %s from %s\n", reqname, pkg_snprintf_s(pkg));
                     continue;
                 }
                 
@@ -493,8 +510,7 @@ static int process_deps(struct pkgset *ps, tn_array *pkgs,
                         if (nmatched == 0 || one_is_marked(matches, nmatched)){
                             msg_i(2, nloop, " %s satisfied by install set\n",
                                   capreq_snprintf_s(req));
-                            if (reqnover)
-                                n_hash_insert(upg->depcache, reqname, (void*)1);
+                            n_hash_insert(upg->depcache, reqname, req);
                             continue;
                             
                        } else if (upg->inst->flags & INSTS_FOLLOW) {
@@ -508,55 +524,85 @@ static int process_deps(struct pkgset *ps, tn_array *pkgs,
                                     upg->uninst_dbpkgs)) {
                     msg_i(2, nloop, " %s satisfied by db\n",
                           capreq_snprintf_s(req));
-                    if (reqnover)
-                        n_hash_insert(upg->depcache, reqname, (void*)1);
+                    n_hash_insert(upg->depcache, reqname, req);
                 
                 } else if (tomark) {
-                    if (how == PROCESS_DEPS) {
-                        if (verbose > 1) {
-                            msg_i(2, nloop, " %s marks %s (cap %s)\n",
-                                  pkg_snprintf_s(pkg), pkg_snprintf_s0(tomark),
-                                  capreq_snprintf_s(req));
-                        } else if (verbose) {
-                            msg(1, "%s marks %s (cap %s)\n",
-                                pkg_snprintf_s(pkg), pkg_snprintf_s0(tomark), 
-                                capreq_snprintf_s(req));
-                        }
+                    char *prefix = "";
                     
-                        if (!is_installable(upg->inst->db, tomark,
-                                            upg->uninst_dbpkgs, 
-                                            upg->inst)) {
-                            upg->nfatal_err++; 
-                            ndepadds = 0;
-                            goto l_end;
-                        }
+                    if (how == PROCESS_ORPHANS)
+                        prefix = "orphaned ";
                     
-                        pkg_dep_mark(tomark);
-                        n_array_push(markarr[nmarkarr], tomark);
-                        nmarked++;
-                        upg->ndep++;
-                        upg->ninstall++;
-                        ndepadds++;
+                    if (verbose > 1) {
+                        msg_i(2, nloop, "%s%s marks %s (cap %s)\n", prefix, 
+                              pkg_snprintf_s(pkg), pkg_snprintf_s0(tomark),
+                              capreq_snprintf_s(req));
+                    } else if (verbose) {
+                        msg(1, " %s%s marks %s (cap %s)\n", prefix, 
+                            pkg_snprintf_s(pkg), pkg_snprintf_s0(tomark), 
+                            capreq_snprintf_s(req));
                     }
                     
-                    if (reqnover)
-                        n_hash_insert(upg->depcache, reqname, (void*)1);
+                    if (!is_installable(upg->inst->db, tomark,
+                                        upg->uninst_dbpkgs, 
+                                            upg->inst)) {
+                        upg->nfatal_err++; 
+                        ndepadds = 0;
+                        goto l_end;
+                    }
+                    
+                    pkg_dep_mark(tomark);
+                    n_array_push(markarr[nmarkarr], tomark);
+                    nmarked++;
+                    upg->ndep++;
+                    upg->ninstall++;
+                    ndepadds++;
+                    
+                    n_hash_insert(upg->depcache, reqname, req);
                     
                 } else {
-                    if (how == PROCESS_DEPS) 
+                    if (how == PROCESS_DEPS) {
                         log(LOGERR, "%s: req %s not found\n",
                             pkg_snprintf_s(pkg), capreq_snprintf_s(req));
-                    else if (how == PROCESS_ORPHANS)
-                        log(LOGERR, "%s is required by %s\n", 
-                            capreq_snprintf_s(req), pkg_snprintf_s(pkg));
-                    else
-                        n_assert(0);
-                    pkg_set_badreqs(pkg);
-                    upg->ndep_err++;
+                        pkg_set_badreqs(pkg);
+                        upg->ndep_err++;
+
+                        
+                    } else if (how == PROCESS_ORPHANS) {
+                        int not_found = 1;
+                        
+                        if ((upg->inst->flags & INSTS_GREEDY)) {
+                            struct pkg *p;
+                            
+                            if ((p = find_pkg(ps->pkgs, pkg->name))) {
+                                if (!pkg_is_marked(p)) {
+                                    msg_i(1, nloop, " greedy upgrade %s "
+                                          "to %s-%s (unresolved %s)\n",
+                                          pkg_snprintf_s(pkg), p->ver, p->rel, 
+                                          capreq_snprintf_s(req));
+                                          
+                                    pkg_dep_mark(p);
+                                    n_array_push(markarr[nmarkarr], p);
+                                    nmarked++;
+                                    upg->ndep++;
+                                    upg->ninstall++;
+                                    ndepadds++;
+                                }
+                                not_found = 0;
+                            }
+                        }
+                        
+                        if (not_found) {
+                            log(LOGERR, "%s is required by %s\n", 
+                                capreq_snprintf_s(req), pkg_snprintf_s(pkg));
+                            pkg_set_badreqs(pkg);
+                            upg->ndep_err++;
+                        }
+                    }
                 }
             }
         }
-
+        
+                
         for (i=0; i<n_array_size(markarr[nmarkarr]); i++) 
             n_array_push(upg->install_pkgs, n_array_nth(markarr[nmarkarr], i));
         
@@ -568,6 +614,7 @@ static int process_deps(struct pkgset *ps, tn_array *pkgs,
         nloop++;
     }
     
+    
  l_end:
     
     n_array_free(markarr[0]);
@@ -580,49 +627,8 @@ static int process_deps(struct pkgset *ps, tn_array *pkgs,
 static
 int add_obsoleted_pkgs(const struct pkg *pkg, struct upgrade_s *upg) 
 {
-    struct capreq *self_cap;
-    int i, k, n = 0;
-    int j, idx;
-    
-    idx = k = n_array_size(upg->uninst_dbpkgs);
-    self_cap = capreq_new(pkg->name, 0, NULL, NULL, 0, 0);
-    rpm_get_obsoletedby_cap(upg->inst->db->dbh, upg->uninst_dbpkgs, self_cap,
-                            PKG_LDWHOLE);
-    capreq_free(self_cap);
-    n = n_array_size(upg->uninst_dbpkgs) - k;
-
-    if (n) {
-        for (j=idx; j<n_array_size(upg->uninst_dbpkgs); j++)
-            msg(1, "%s obsoleted by %s\n",
-                dbpkg_snprintf_s(n_array_nth(upg->uninst_dbpkgs, j)),
-                pkg_snprintf_s(pkg));
-    }
-    
-    if (pkg->cnfls == NULL)
-        return n;
-    
-    k = n_array_size(upg->uninst_dbpkgs);
-    
-    for (i=0; i < n_array_size(pkg->cnfls); i++) {
-        struct capreq *cnfl = n_array_nth(pkg->cnfls, i);
-        
-        
-        if (!cnfl_is_obsl(cnfl))
-            continue;
-
-        idx = n_array_size(upg->uninst_dbpkgs);
-        if (rpm_get_obsoletedby_cap(upg->inst->db->dbh, upg->uninst_dbpkgs,
-                                    cnfl, PKG_LDWHOLE)) {
-            
-            for (j=idx; j<n_array_size(upg->uninst_dbpkgs); j++)
-                msg(1, "%s obsoleted by %s\n",
-                    dbpkg_snprintf_s(n_array_nth(upg->uninst_dbpkgs, j)),
-                    pkg_snprintf_s(pkg));
-        }
-    }
-
-    n += n_array_size(upg->uninst_dbpkgs) - k;
-    return n;
+    return rpm_get_obsoletedby_pkg(upg->inst->db->dbh, upg->uninst_dbpkgs, pkg,
+                                   PKG_LDWHOLE);
 }
 
 /* add to upg->orphan_dbpkgs packages which are required by uninst_dbpkgs */
@@ -886,7 +892,7 @@ int find_db_conflicts2(const struct pkg *pkg, const struct capreq *cap,
         
         for (j=0; j<n_array_size(dbpkg->pkg->cnfls); j++) {
             struct capreq *cnfl = n_array_nth(dbpkg->pkg->cnfls, j);
-            if (cap_match_req(cap, cnfl, 0))
+            if (cap_match_req(cap, cnfl, 1))
                 log(LOGERR, "%s (%s) conflicts with installed %s (%s)\n",
                     pkg_snprintf_s(pkg), capreq_snprintf_s(cap), 
                     pkg_snprintf_s0(dbpkg->pkg),capreq_snprintf_s0(cnfl));
@@ -990,7 +996,65 @@ static int valid_arch_os(tn_array *pkgs)
     return nerr == 0;
 }
 
+
+static void print_install_summary(struct upgrade_s *upg) 
+{
+    int i;
     
+    msg(1, "There are %d package%s to install", upg->ninstall,
+        upg->ninstall > 1 ? "s":"");
+
+    
+    if (upg->ndep) 
+        msg(1, " (%d marked by dependencies)", upg->ndep);
+
+    if (n_array_size(upg->uninst_dbpkgs))
+        msg(1, ", %d to uninstall", n_array_size(upg->uninst_dbpkgs));
+    msg(1, ":\n");
+    
+    
+    for (i=0; i<n_array_size(upg->install_pkgs); i++) {
+        struct pkg *pkg = n_array_nth(upg->install_pkgs, i);
+        msg(1, "%c %s\n", pkg_is_dep_marked(pkg) ? 'D' : 'I',
+            pkg_snprintf_s(pkg));
+    }
+
+    for (i=0; i<n_array_size(upg->uninst_dbpkgs); i++) {
+        struct dbpkg *dbpkg = n_array_nth(upg->uninst_dbpkgs, i);
+        msg(1, "R %s\n", pkg_snprintf_s(dbpkg->pkg));
+    }
+    
+}
+
+
+static int check_holds(struct pkgset *ps, struct upgrade_s *upg)
+{
+    int i, j, rc = 1;
+
+    if ((ps->flags & (PSMODE_UPGRADE | PSMODE_UPGRADE_DIST)) == 0)
+        return 1;
+
+    if (upg->inst->hold_pkgnames == NULL)
+        return 1;
+
+    for (i=0; i < n_array_size(upg->uninst_dbpkgs); i++) {
+        struct dbpkg *dbpkg = n_array_nth(upg->uninst_dbpkgs, i);
+
+        for (j=0; j<n_array_size(upg->inst->hold_pkgnames); j++) {
+            const char *mask = n_array_nth(upg->inst->hold_pkgnames, j);
+
+            if (fnmatch(mask, dbpkg->pkg->name, 0) == 0) {
+                log(LOGERR, "%s: refusing to uninstall held package\n",
+                    pkg_snprintf_s(dbpkg->pkg));
+                rc = 0;
+            }
+        }
+    }
+    
+    return rc;
+}
+
+
 /* process packages to install:
    - check dependencies
    - mark unresolved dependencies finded in pkgset
@@ -999,7 +1063,7 @@ static int valid_arch_os(tn_array *pkgs)
 static
 int pkgset_do_install(struct pkgset *ps, struct upgrade_s *upg)
 {
-    int i, ncnfl = 0, ndbcnfl = 0, rc;
+    int ncnfl = 0, ndbcnfl = 0, rc, nerr = 0;
     
     msg(1, "Processing dependencies...\n");
 
@@ -1008,47 +1072,30 @@ int pkgset_do_install(struct pkgset *ps, struct upgrade_s *upg)
     if (upg->nfatal_err)
         return 0;
 
-    if (upg->ndep == 0)
-        msg(1, "There are %d package%s to install:\n", upg->ninstall,
-            upg->ninstall > 1 ? "s":"");
-    else 
-        msg(1, "There are %d package%s to install, "
-            "%d marked by dependencies:\n", upg->ninstall,
-            upg->ninstall > 1 ? "s":"", upg->ndep);
-        
-    for (i=0; i<n_array_size(upg->install_pkgs); i++) {
-        struct pkg *pkg = n_array_nth(upg->install_pkgs, i);
-        msg(1, "_%c %s\n", pkg_is_dep_marked(pkg) ? 'D' : 'I', 
-            pkg_snprintf_s(pkg));
-    }
-    
+    print_install_summary(upg);
     msg(1, "Verifying conflicts...\n");
     ncnfl = find_conflicts(upg, &ndbcnfl);
     
-    if (ndbcnfl) {
-        log(LOGERR, "There are conflicts in install set, give up\n");
-        return 0;
+    if (ncnfl) {
+        log(LOGERR, "There %d are conflicts\n", ncnfl);
+        
+        if (upg->inst->instflags & (PKGINST_FORCE | PKGINST_TEST)) 
+            ncnfl = ndbcnfl = 0;
+        else
+            nerr++;
     }
-
-    /* insert check conflicts against db there ... */
-    if (upg->inst->instflags & PKGINST_FORCE) {
-        ncnfl = 0;
-        ndbcnfl = 0;
-    }
-
-    if (ndbcnfl && (upg->inst->instflags & PKGINST_TEST) == 0) {
-        log(LOGERR, "Stop\n"); 
-        return 0;
-    }
-
+    
     if (upg->ndep_err) {
         log(LOGERR, "There are %d unresolved dependencies.\n", upg->ndep_err);
-        if ((upg->inst->instflags & (PKGINST_TEST | PKGINST_NODEPS)) == 0)
-            return 0;
-        
-        if (upg->inst->instflags & PKGINST_NODEPS)
+        if (upg->inst->instflags & (PKGINST_NODEPS | PKGINST_TEST))
             upg->ndep_err = 0;
+        else 
+            nerr++;
     }
+    
+    if (nerr)
+        return 0;
+    
 
     if ((upg->inst->flags & (INSTS_JUSTPRINT | INSTS_JUSTFETCH)) == 0)
         if (!valid_arch_os(upg->install_pkgs)) 
@@ -1068,7 +1115,7 @@ int pkgset_do_install(struct pkgset *ps, struct upgrade_s *upg)
         n_assert(upg->inst->fetchdir);
         rc = pkgset_fetch_pkgs(upg->inst->fetchdir, upg->install_pkgs, 1);
         
-    } else {
+    } else if ((upg->inst->flags & INSTS_NOHOLD) || (rc=check_holds(ps, upg))) {
         rc = runrpm(ps, upg);
     }
     
@@ -1078,7 +1125,7 @@ int pkgset_do_install(struct pkgset *ps, struct upgrade_s *upg)
 
 /* save in upg->install_pkgs if newer version finded */
 static 
-void mapfn_chk_newer_pkg(unsigned recno, void *h, void *upgptr) 
+void mapfn_check_newer_pkg(unsigned recno, void *h, void *upgptr) 
 {
     struct upgrade_s *upg = upgptr;
     uint32_t *epoch;
@@ -1115,9 +1162,15 @@ void mapfn_chk_newer_pkg(unsigned recno, void *h, void *upgptr)
         }
         
         if (cmprc > 0) {
-            pkg_hand_mark(pkg);
-            n_array_push(upg->install_pkgs, pkg);
-            n_array_push(upg->uninst_dbpkgs, dbpkg_new(recno, h, PKG_LDWHOLE));
+            if (pkg_is_hold(pkg)) {
+                msg(0, "%s: skip held package\n", pkg_snprintf_s(pkg));
+                
+            } else {
+                pkg_hand_mark(pkg);
+                n_array_push(upg->install_pkgs, pkg);
+                n_array_push(upg->uninst_dbpkgs, dbpkg_new(recno, h, PKG_LDWHOLE));
+                upg->ninstall++;
+            }
         }
     }
 }
@@ -1159,26 +1212,26 @@ int pkgset_upgrade_dist(struct pkgset *ps, struct inst_s *inst)
 {
     int rc = 1;
     struct upgrade_s upg;
+    
 
     init_upgrade_s(&upg, ps, inst);
-    
-    
-    msg(1, "_Looking up packages for upgrade...\n");
+
+    msg(1, "Looking up packages for upgrade...\n");
     
     //set_capreq_allocfn(malloc, free, &capreq_alloc_fn, &capreq_free_fn);
 
     /* find packages to upgrade */
-    pkgdb_map(inst->db, mapfn_chk_newer_pkg, &upg);
+    pkgdb_map(inst->db, mapfn_check_newer_pkg, &upg);
     n_array_sort(upg.uninst_dbpkgs);
 
     // set_capreq_allocfn(capreq_alloc_fn, capreq_free_fn, NULL, NULL);
 
     if (upg.ndberrs) {
         log(LOGERR, "There are database errors (?), give up\n");
-        n_array_free(upg.install_pkgs);
+        destroy_upgrade_s(&upg);
         return 0;
     }
-    
+
     if (n_array_size(upg.install_pkgs) == 0) {
         msg(1, "All packages are up to date, nothing to do\n");
         
