@@ -7,7 +7,7 @@
 */
 
 /*
-  $Id: pkgset.c,v 1.18 2001/06/27 16:29:11 mis Exp $
+  $Id: pkgset.c,v 1.20 2001/07/10 15:37:22 mis Exp $
 */
 
 #include <limits.h>
@@ -123,18 +123,22 @@ void pkgsetmodule_destroy(void)
 
 void inst_s_init(struct inst_s *inst)
 {
+    inst->flags = INSTS_FOLLOW;
     inst->db = NULL;
+    
     inst->instflags = 0;
     inst->rootdir = NULL;
     inst->fetchdir = NULL;
-    inst->cachedir = NULL;
+    inst->cachedir = tmpdir();
     inst->dumpfile = NULL;
     inst->rpmopts = NULL;
     inst->rpmacros = NULL;
-    
     inst->selpkg_fn = NULL;
     inst->ask_fn = NULL;
     inst->inf_fn = NULL;
+    inst->rpmacros = n_array_new(2, NULL, NULL);
+    inst->rpmopts = n_array_new(4, NULL, (tn_fn_cmp)strcmp);
+    inst->hold_pkgnames = n_array_new(4, free, (tn_fn_cmp)strcmp);
 }
 
 
