@@ -1,5 +1,5 @@
 /* 
-  Copyright (C) 2000 Pawel A. Gajda (mis@k2.net.pl)
+  Copyright (C) 2000, 2001 Pawel A. Gajda (mis@k2.net.pl)
  
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License published by
@@ -7,7 +7,7 @@
 */
 
 /*
-  $Id: rpm.c,v 1.15 2001/06/22 17:18:37 mis Exp $
+  $Id: rpm.c,v 1.17 2001/07/10 15:37:22 mis Exp $
 */
 
 #ifdef HAVE_CONFIG_H
@@ -303,25 +303,32 @@ int rpm_dbmatch_req(rpmdb db, const struct capreq *req, int strict,
  */
 static void progress(const unsigned long amount, const unsigned long total) 
 {
-    static unsigned long prev_v = 0, vv = 0;
-    
-    if (amount && amount == total) { /* last notification */
-        msg(1, "_. (%ld kB)\n", total/1024);
-
-    } else if (amount == 0) {     /* first notification */
-        msg(1, "_.");
-        vv = 0;
-        prev_v = 0;
+    if (amount == 0) {     /* first notification */
+        msg(0, ".");
         
-    } else if (total == 0) {     /* impossible */
-        assert(0);
-
     } else {
-        unsigned long i;
-        unsigned long v = amount * 60 / total;
-        for (i=prev_v; i<v; i++)
-            msg(1, "_.");
-        prev_v = v;
+        char   line[256], outline[256], fmt[40];
+        float  frac, percent;
+        int    barwidth = 75, n;
+        
+
+        frac = (float) amount / (float) total;
+        percent = frac * 100.0f;
+        
+        barwidth -= 7;
+        n = (int) (((float)barwidth) * frac);
+        
+        memset(line, '.', n);
+        line[n] = '\0';
+        snprintf(fmt, sizeof(fmt), "%%-%ds %%5.1f%%%%", barwidth);
+        snprintf(outline, sizeof(outline), fmt, line, percent);
+        
+        if (amount && amount == total) { /* last notification */
+            msg(0, "\r%s\n", outline);
+            
+        } else {
+            msg(0, "\r%s", outline);
+        }
     }
 }
 
@@ -346,7 +353,7 @@ static void *install_cb(const void *h __attribute__((unused)),
             break;
 
         case RPMCALLBACK_INST_START:
-            msg(1, "Installing %s\n", n_basenam(pkgpath));
+            msg(0, "Installing %s\n", n_basenam(pkgpath));
             progress(amount, total);
             break;
 
@@ -372,7 +379,7 @@ int rpm_install(rpmdb db, const char *rootdir, const char *path,
     int rc;
     Header h = NULL;
 
-
+    
     if (rootdir == NULL)
         rootdir = "";
 
@@ -554,8 +561,8 @@ int rpm_get_pkgs_requires_capn(rpmdb db, tn_array *dbpkgs, const char *capname,
     return n;
 }
 
-int rpm_get_obsoletedby_cap(rpmdb db, tn_array *dbpkgs, struct capreq *cap,
-                            unsigned ldflags)
+int rpm_get_obsoletedby_cap(rpmdb db, tn_array *dbpkgs, const struct pkg *pkg,
+                            struct capreq *cap, unsigned ldflags)
 {
     struct rpmdb_it it;
     const struct dbrec *dbrec;
@@ -567,8 +574,11 @@ int rpm_get_obsoletedby_cap(rpmdb db, tn_array *dbpkgs, struct capreq *cap,
 	    continue;
         
         if (header_evr_match_req(dbrec->h, cap)) {
-            n_array_push(dbpkgs, dbpkg_new(dbrec->recno, dbrec->h, ldflags));
+            struct dbpkg *dbpkg = dbpkg_new(dbrec->recno, dbrec->h, ldflags);
+            n_array_push(dbpkgs, dbpkg);
             n_array_sort(dbpkgs);
+            msg(1, " %s obsoleted by %s (cap %s)\n", dbpkg_snprintf_s(dbpkg), 
+                pkg_snprintf_s(pkg), capreq_snprintf_s(cap));
             n++;
         }
     }
@@ -576,6 +586,35 @@ int rpm_get_obsoletedby_cap(rpmdb db, tn_array *dbpkgs, struct capreq *cap,
     return n;
 }
 
+
+int rpm_get_obsoletedby_pkg(rpmdb db, tn_array *dbpkgs, const struct pkg *pkg,
+                            unsigned ldflags)
+{
+    struct capreq *self_cap;
+    int i, n = 0;
+    
+
+    self_cap = capreq_new(pkg->name, 0, NULL, NULL, 0, 0);
+    n = rpm_get_obsoletedby_cap(db, dbpkgs, pkg, self_cap, ldflags);
+    capreq_free(self_cap);
+    
+    if (pkg->cnfls == NULL)
+        return n;
+    
+    for (i=0; i < n_array_size(pkg->cnfls); i++) {
+        struct capreq *cnfl = n_array_nth(pkg->cnfls, i);
+
+        if (!cnfl_is_obsl(cnfl))
+            continue;
+
+        n += rpm_get_obsoletedby_cap(db, dbpkgs, pkg, cnfl, ldflags);
+    }
+    
+    return n;
+}
+
+
+
 static 
 int hdr_pkg_cmp_evr(Header h, const struct pkg *pkg)
 {
