@@ -7,7 +7,7 @@
 */
 
 /*
-  $Id: desc.c,v 1.4 2001/06/26 19:55:58 mis Exp $
+  $Id: desc.c,v 1.5 2001/07/09 15:12:53 mis Exp $
 */
 
 #include <time.h>
@@ -189,7 +189,7 @@ static void show_caps(struct pkg *pkg)
                 nlident(ncol);
             }
             
-            if (i + 1 == ncaps)
+            if (--ncaps == 0)
                 colon = "";
             
             ncol += printf("%s%s", p, colon);
